/* $Id: sylcd.c,v 1.3 2003/05/02 12:35:10 dijkstra Exp $ */

/*
 * Copyright (c) 2003 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sysexits.h>

#include "error.h"
#include "lcd.h"
#include "net.h"
#include "st_if.h"
#include "sylcd.h"
#include "stream.h"
#include "xmalloc.h"

void
init_streams(struct streamlist *sl, char *stream_host, char *s, size_t maxlen) 
{
    int i;
    struct stream *st;
    char *end;

    while ((i = get_stream_index(s, maxlen, "if", NULL)) != -1) {

	/* jump over 'if:' */
	i += 3;
	maxlen -= i;
	s += i;

	/* seek end of if definition 'xl0:' */
	if ((end = memchr(s, ':', maxlen)) == NULL)
	    return;

	*end = '\0';

	if (find_stream(sl, stream_host, "if", s) == NULL) {
	    if ((st = create_stream(stream_host, "if", s)) == NULL)
		fatal("%s:%d:could not create stream", __FILE__, __LINE__);

	    debug("found interface %s", s);

	    SLIST_INSERT_HEAD(sl, st, next);
	}

	/* jump over end of string and rerun */
	i = (end - s) + 1;
	maxlen -= i;
	s += i;
    }
}

int 
main(int argc, char** argv) 
{
    struct muxconn m;
    struct streamlist sl;
    char *stream_host, *mux_host, *mux_port, *serial, *s;
    int i, sleeper;
    char ch;
    size_t maxlen;
    struct stream *st, *show_st;

    flag_debug = 0;
    flag_daemon = 0;
    stream_host = DEFAULT_SOURCE_HOST;
    mux_host = DEFAULT_MUX_HOST;
    mux_port = DEFAULT_MUX_PORT;
    serial = DEFAULT_SERIAL_LINE;

    sleeper = 0;
    st = show_st = NULL;
    SLIST_INIT(&sl);
    
    while ((ch = getopt(argc, argv, "dvH:")) != -1) {
	switch (ch) {
	case 'd':
	    flag_debug = 1;
	    break;

	case 'H':
	    if (optarg) {
		stream_host = xstrdup(optarg);
	    } else {
		fatal("Need a <host to display>");
	    }
	    break;

	case 's':
	    if (optarg) {
		serial = xstrdup(optarg);
	    } else {
		fatal("Need a <serial>");
	    }
	    break;

	case 'v':
	    info("sylcd version " SYLCD_VERSION " (" LCD_NAME " / " LCD_FORMAT ")");
	    /* drop through */
	default:
	    info("usage: %s \t[-d] [-v] [-H <host to display>]\n"
		 "\t\t[-s <serial>] [<symux host>] [<symux port>]\n", __progname);
	    info("\t-d                     debug, also prevents becoming a daemon");
	    info("\t-v                     version information");
	    info("\t-H <host to display>   which host data should be displayed");
	    info("\t                       default: " DEFAULT_SOURCE_HOST);
            info("\t-s <serial>            serial line attached to lcd");
            info("\t                       default: " DEFAULT_SERIAL_LINE);
            info("\t<symux host>           symux to connect to");
            info("\t                       default: " DEFAULT_MUX_HOST);
            info("\t<symux port>           symux port to connect to");
	    info("\t                       default: " DEFAULT_MUX_PORT);
					     
	    exit(EX_USAGE);
	    break;
	}
    }

    argc -= optind;
    argv += optind;

    switch (argc) {
    case 2:
	mux_host = xstrdup(*argv);
	argv++;
	mux_port = xstrdup(*argv);
	break;
    case 1:
	mux_host = xstrdup(*argv);
	break;
    case 0:
	break;
    default:
	fatal("Too many arguments");
    }

    setegid(getgid());
    setgid(getgid());

    if (flag_debug != 1) {
	if (daemon(0, 0) != 0)
	    fatal("daemonize failed");

	flag_daemon = 1;
    }

    lcd_init(serial, LCD_VBAR);
    lcd_printf(0, 0, "sylcd " SYLCD_VERSION);
    lcd_flush();

    bzero(&m, sizeof(m));
    init_muxconn(&m, mux_host, mux_port);
    
    debug("detecting interfaces in %s host streams", stream_host);

    for (i = 0; i < SYMON_INTERVAL * 2; i++) {
	lcd_printf(0, 1, "wakeup in %d s ", (SYMON_INTERVAL *2) - i);
	lcd_flush();
	sleep(1);
    }
    
    while (get_muxdata(&m) == (size_t) 0)
	lcd_printf(0, 1, ".");
	sleep(1);
    
    while (get_line(&m, &s, &maxlen)) {
	if (!is_host_stream(s, maxlen, stream_host))
	    break;
	
	init_streams(&sl, stream_host, s, maxlen);
    }

    if (SLIST_EMPTY(&sl))
	fatal("no interfaces found for %s in streams from %s %s", 
	      stream_host, mux_host, mux_port);

    SLIST_FOREACH(st, &sl, next) {
	if_init(st);
    }

    info("sylcd " SYLCD_VERSION);
    show_st = SLIST_FIRST(&sl);

    for (;;) {
	if (get_muxdata(&m)) {
	    while (get_line(&m, &s, &maxlen)) {

		if (!is_host_stream(s, maxlen, stream_host))
		    break;

		SLIST_FOREACH(st, &sl, next)
		    if ((i = get_stream_index(s, maxlen, st->stype, st->sargs)) > 0)
			if_read(st, &s[i], maxlen - i);
	    }

	    
	    if_show(show_st, IFMODE_INOUT);
	    lcd_flush();

	    sleeper++;
	    if (sleeper > 3) {
		if (show_st == NULL || 
		    SLIST_NEXT(show_st, next) == SLIST_END(show_st))
		    show_st = SLIST_FIRST(&sl);
		else
		    show_st = SLIST_NEXT(show_st, next);

		sleeper = 0;
	    }
	}
	sleep(1);
    }
}
